function Edges = EdgeExtraction(I,VeinFraction,MajorAxisLength,EulerNumber,MinimumBranchLength)
% [Briefe explanation of this function in a few rows is here.]
% 
% Inputs:
%   - I: an uwrapped CT image
%   - VeinFraction: maximum vein fraction for binarization [%]
%   - MajorAxisLength: length (in pixels) of the major axis of the ellipse that has the same normalized second central moments as the region
%   - EulerNumber: number of objects in the region minus the number of holes in those objects
%   - MinimumBranchLength: minimum branch length for pruning
% 
% Output:
%   - Edges that are likely parts of veins
%     (z=1: Low-CT veins, z=2: High-CT veins, z=3: empty)

meanCT = mean(I(:)); % Mean CT number of the core

% Extract area with CT numbers lower/higher than the meanCT
BW_low = I >= 0 & I < meanCT; 
BW_high = I >= meanCT;

% Create CT image with CT number less/higher than the meanCT
CT_low = I .* (I >= 0 & I < meanCT); 
CT_high = I .* (I >= meanCT);

% Replace the margin area of I_low/I_high with the meanCT
% so that they are not detected as edges
CT_low(BW_low == 0) = meanCT;
CT_high(BW_high == 0) = meanCT;

% Apply the Jerman filter to extract vein-like features
Jerman_low = vesselness(CT_low,0.5:0.5:1.5,[1;1],1,false);
Jerman_high = vesselness(CT_high,0.5:0.5:1.5,[1;1],1,true);

% Binarize based on percentile method
BW_low1 = logical(ptile(Jerman_low,100 - VeinFraction));
BW_high1 = logical(ptile(Jerman_high,100 - VeinFraction));

% Remove the connected components whose major axis length < threshold
BW_low2 = bwpropfilt(BW_low1,'MajorAxisLength',[MajorAxisLength,inf]);
BW_high2 = bwpropfilt(BW_high1,'MajorAxisLength',[MajorAxisLength,inf]);

% Remove the connected components whose Euler number < threshold
BW_low3 = bwpropfilt(BW_low2,'EulerNumber',[EulerNumber,1]);
BW_high3 = bwpropfilt(BW_high2,'EulerNumber',[EulerNumber,1]);

% Skeletonize
BW_low4 = bwmorph(BW_low3,'skel',inf);
BW_high4 = bwmorph(BW_high3,'skel',inf);

% Prune small spurs that appear on the skeleton
BW_low5 = bwskel(BW_low4,'MinBranchLength',MinimumBranchLength);
BW_high5 = bwskel(BW_high4,'MinBranchLength',MinimumBranchLength);

% Obtain the binary images
Edges = zeros(size(I,1),size(I,2),3);
Edges(:,:,1) = BW_low5; % for Low-CT veins
Edges(:,:,2) = BW_high5; % for High-CT veins

%% Showing results of edge extraction
[height, width] = size(I); % Size of image
lengthpix = 0:height-1; % Number of CT cross sections
lengthmm = 0.625*lengthpix; % Total length in mm of core CT scan (0.625 mm/px)   
azimuth = linspace(0,360,width); % Apparent azimuth

f1 = figure();
f1.Units = 'centimeters';
f1.Position(3:4) = [15 20];
t1 = tiledlayout(1,2);

ax1(1) = nexttile;
imagesc(azimuth,lengthmm,I)
xlabel('Azimuth (\circ)');
ylabel('Length (mm)');
xticks([0,90,180,270,360])
colormap(ax1(1),'parula')
c = colorbar('northoutside');
c.Label.String = 'CT number';
clim([1000 4000]); 
set(gca,'YDir','reverse')

% Candidate edges for High-CT/Low-CT veins are shown in red and green,
% respectively.
ax1(2) = nexttile;   
imagesc(azimuth,lengthmm,Edges)
xlabel('Azimuth (\circ)');
ylabel('Length (mm)');
xticks([0,90,180,270,360])
set(gca,'YDir','reverse')