function [A_hough, theta_hough, maximum] = HoughSineCurve(I, y_offset)
% Performs tow-dimensional Hough transform for detecting a since curve in
% an unwrapped core image with a pre-determined offset y0.
% 
% [Inputs]
% I      : Binary input image to be processed
% y_offset: Pre-determined offset

% [Outputs]
% A_hough    : Amplitude of the detected sine wave
% theta_hough: Phase of the detected sine wave
% maximum    : The highest value in the parameter space

[~, width] = size(I);

% Find all edge positions within the binary image
[edge_rows, edge_cols] = find(I);

% Define the range of the amplitude and phase values
amp_min = 0;
amp_max = 300;
vector_amp = linspace(amp_min, amp_max, amp_max - amp_min + 1);
vector_phase = 2 * pi / width * linspace(0, width, 361);

% Initialize the Hough space
H = zeros([length(vector_phase), length(vector_amp), length(y_offset)]);

% Vote for each edge pixel
for i = 1:length(edge_cols)
    for j = 1:length(vector_phase)
        amp_ij = round((edge_rows(i) - y_offset) ./ sin(2 * pi / width * edge_cols(i) + vector_phase(j)));
        
        if 0 <= amp_ij && amp_ij < amp_max
            index_amp = find(vector_amp >= amp_ij, 1, 'first');
            H(j, index_amp) = H(j, index_amp) + 1;
        end
    end
end

% Find the maximum value in the Hough space
maximum = max(max(H));
[theta_index, amp_index] = find(H == maximum, 1, 'first');

% Retrieve the amplitude and phase values corresponding to the maximum value in H
A_hough = vector_amp(amp_index);
theta_hough = vector_phase(theta_index);