%% Sine curve detection in XCT core images%%

clear
close all

filename = 'UI_GT3_036Z02.mat';
load(filename);

%% Parameter setting

% Threshold value used for pre- and post-processing
VeinFraction = 6; % Fraction of vein [%]
MajorAxisLength = 40; % Length (in pixels) of the major axis of the ellipse that has the same normalized second central moments as the region
EulerNumber = -10; % Number of objects in the region minus the number of holes in those objects
MinimumBranchLength = 10; % Minimum branch length for pruning

% Parameters used for Hough transform
WindowSize = 50; % window size for offset (y0) detection
MinimumPeakProminece = 1 ; % Mimimum Peak Prominence for accumulator of offset detection
MinimumPeakDistance = 10; % Mimimum Peak Distance for accumulator of offset detection
VeinRegionWidth = 10; % width of search region along detected sine curves

% Threshold used for verlification of positive
SpreadingThreshold = 376;

%% Edge extraction

CT_cut = cutImageMargins(unwrap); % Cut blank areas below and above the CT image

[height, width] = size(CT_cut); % Size of image
lengthpix = 0:height-1; % Number of CT cross sections  
lengthmm = 0.625*lengthpix; % Total length in mm of core CT scan (0.625 mm/px)   
azimuth = linspace(0,360,width); % Apparent azimuth

meanCT = mean(CT_cut(:)); % Mean CT number of the core

Edges = EdgeExtraction(CT_cut,VeinFraction,MajorAxisLength,EulerNumber,MinimumBranchLength);

%% Iterative application of two-step Hough transform

% Preallocation
A = []; Theta = []; Y0 = []; Aperture = []; VeinCTmean = []; VeinCT1000 = []; skip_y0 = []; 
VeinType = []; 
% 1:Low-CT, 2:High-CT, 3:Fracture

for i = 1:2

    I = Edges(:,:,i);
    
    x = 1:1:width;
    N_initial = length(find(I)); % Number of edge pixels
    N = N_initial;
    N_threshold = round(width/2);    

    % Iteration while the number of edges is larger than the threshold
    while N > N_threshold

        % Offset value is determined by a search strategy based on Changchun & Ge (2002) method
        [y0,pmax] = y0Detection(I,WindowSize,skip_y0,MinimumPeakProminece,MinimumPeakDistance); 

        % If there is no candidate peak, finish the loop.
        if pmax == 0
            N = 0;
        else
            % 2-D Hough transform
            [a,theta,vote] = HoughSineCurve(I,y0);

            y = round(a * sin(2 * pi/width * x + theta) + y0); % Sine curve determined by the Hough transform

            % Set ROI for thresholding and calculating CT number and
            % aperture
            y_above = y - VeinRegionWidth; % Top of ROI
            y_above(y_above < 1) = 1;
            y_below = y + VeinRegionWidth; % Bottom of ROI
            y_below(y_below > height) = height;

            % Search region along detected sine curve
            ROI = zeros(size(I)); 
            for i2 = 1:width
                ROI(y_above(i2):y_below(i2),x(i2)) = 1;
            end

            % Binary images within ROI
            I_ROI = I .* ROI;         
            
            % Calculate "Spreading"; if it is less than the threshold, 
            % this is a false positive.

            spread = getEdgeSpread(I_ROI);             
            
            if spread > SpreadingThreshold
                
                CT_ROI = CT_cut .* ROI;
                
                % Zero out the skeleton image around the curve
                % and calculate CT number of edge pixels
                I_vein = I .* ROI .* CT_cut;
                for i3 = 1:width                                  
                    veinCT(i3) = I_vein(y(i3),x(i3)); % CT number of each vein pixel
                    I(y_above(i3):y_below(i3),x(i3)) = 0;                                
                end
                
                veinCT = veinCT(veinCT~=0);
                
                % Mean/standard deviation 
                veinCTmean = mean(veinCT); % Mean value of vein CT numbers
                veinCTstd  = std(veinCT); % Standard deviation of vein CT numbers
                
                % Fraction of low CT pixels                 
                veinCT1000 = length(veinCT(veinCT<1000))/length(veinCT);
                
                % Calculation of the vein aperture
                vein_area = length(CT_ROI(CT_ROI>veinCTmean - veinCTstd & CT_ROI<veinCTmean + veinCTstd));
                arclen = getSineArcLength(a,theta,width); % Arc length of sine curve [px]
                aperture = vein_area/arclen; % apparent width of vein [px]

                % If the aperture is too large, it is probably a false positive.
                if aperture <= 10

                % If the fraction of low CT pixel is larger than 0.01 (1%), it is defined as a fracture.
                    if veinCT1000 > 0.01
                        veintype = 3; % Fracture
                    else
                        veintype = i; % Vein
                    end
                                    
                    % Save parameters
                    VeinType = [VeinType;veintype];
                    A = [A;a];
                    Theta = [Theta;theta];
                    Y0 = [Y0;y0];
                    Aperture = [Aperture;aperture];
                    VeinCTmean = [VeinCTmean;veinCTmean];

                end
            end

            skip_y0 = [skip_y0;y0];     

            N = length(find(I)); % Number of edges in the skeleton image

            % If the number of trial exceeds 30, the detection is likely
            % failed... Please change the setting parameters.
            if length(skip_y0) == 30
                N = 0;
            end 
        end
    end
end
%% Results
% Summary of results
parameters = table(VeinType,Y0,A,Theta,Aperture,VeinCTmean);
parameters = sortrows(parameters,'Y0');

k1 = find(VeinType == 1); % Low-CT veins
k2 = find(VeinType == 2); % High-CT veins
k3 = find(VeinType == 3); % Fractures

if isempty(k1) == 0
    Y1 = round(A(k1) .* sin(2*pi/width * x + Theta(k1)) + Y0(k1));
else
    Y1 = 0;
end

if isempty(k2) == 0
    Y2 = round(A(k2) .* sin(2*pi/width * x + Theta(k2)) + Y0(k2));
else
    Y2 = 0;
end

if isempty(k3) == 0
    Y3 = round(A(k3) .* sin(2*pi/width * x + Theta(k3)) + Y0(k3));
else
    Y3 = 0;
end

% Calculation of strike and dip angles and aperture
r = 2 * 100 * 0.17578125 * 1e-3; % diameter of the cropped core (35.2 mm)

A_mm = 2 * parameters.A * 0.625 * 1e-3; % [mm]
parameters.Strike_deg = rad2deg(parameters.Theta); % Strike [deg]
parameters.Dip_deg = rad2deg(atan(A_mm/r)); % Dip angle [deg]
parameters.Aperture_mm = parameters.Aperture .* cosd(parameters.Dip_deg); % Aperture [mm]

disp(parameters)

%% Plotting results
f2 = figure();
f2.Units = 'centimeters';
f2.Position(3:4) = [15 20];
t2 = tiledlayout(1,2);

ax2(1) = nexttile;
imagesc(CT_cut);hold on
plot(x,Y1,'--','Color','b', 'LineWidth',1);
plot(x,Y2,'--','Color','r', 'LineWidth',1);
plot(x,Y3,'--','Color','w', 'LineWidth',1);
xlabel('Azimuth (px)');
ylabel('Length (px)');
colormap(ax2(1),'parula');
c = colorbar('northoutside');
c.Label.String = 'CT number';
clim([1000 4000]); 
set(gca,'YDir','reverse')

ax2(2) = nexttile;
imagesc(Edges);hold on
plot(x,Y1,'--','Color','b', 'LineWidth',1);
plot(x,Y2,'--','Color','r', 'LineWidth',1);
plot(x,Y3,'--','Color','w', 'LineWidth',1);
xlabel('Azimuth (px)');
ylabel('Length (px)');
colormap(ax2(2),gray);
set(gca,'YDir','reverse')
